package viajante_va;


public class BuscaCircuitoVA {
	
	// habilita o deshabilita los mensajes de depuración (no pedido en el examen)
	private static final boolean DBG = false;
	
	// Cuenta los nodos explorados (no pedido en el examen)
	private static int nodosExplorados = 0;
	
	// Mejor solución encontrada.
	// Este atributo es compartido por todas las llamadas recursivas.
	private static Itinerario mejorSol;

	/**
	 * Resuelve el problema del viajante.
	 * @param distancias matriz de distancias entre las ciudades
	 * @return itinerario que constituye la mejor solución encontrada
	 */
	public static Itinerario buscaCircuitoMasCorto(double[][] distancias) {
		// crea una primera mejor solución
		mejorSol = new Itinerario(distancias, 0);
		for(int i=1; i<distancias.length; i++)
			mejorSol.incluyeCiudad(i);
		
		buscaCircuitoMasCortoRec(distancias, new Itinerario(distancias, 0));
		
		System.out.println("Nodos explorados:"+nodosExplorados);
		return mejorSol;
	}
	
	/**
	 * Busca el circuito más corto teniendo en cuenta que ya se ha recorrido
	 * el itinerario que recibe en el parámetro 'sol'
	 * @param distancias matriz de distancias entre las ciudades
	 * @param sol itinerario ya recorrido hasta este momento
	 */
	private static void buscaCircuitoMasCortoRec(double[][] distancias,
			Itinerario sol) {
		
		// cuestión 3.b)
		if (sol.distanciaRecorrida() >= mejorSol.distanciaRecorrida()) {
			// no merece la pena seguir a partir de un nodo que ya no
			// puede mejorar la mejor solución encontrada
			return;
		}
		
		nodosExplorados++; // no pedido en el examen
		
		// si se han visitado todas las ciudades, hemos encontrado una solución
		if (sol.recorridoCompleto() && 
				sol.distanciaRecorrida() < mejorSol.distanciaRecorrida()) {
			// si es mejor que la mejor solución encontrada hasta ahora, se
			// convierte en la nueva mejor solución
			mejorSol = sol.copia();
			if (DBG) System.out.println("mejor sol:"+mejorSol);
		}
		
		// realiza una llamada recursiva para cada ciudad que no haya sido
		// visitada aún (no está en 'sol')
		for(int i=0; i<distancias.length; i++) {
			if (sol.ciudadYaIncluida(i))
				continue;
			
			// añade la ciudad al camino
			sol.incluyeCiudad(i);
			if (DBG) System.out.println("incluye:"+i);
			
			// hace una llamada recursiva
			buscaCircuitoMasCortoRec(distancias, sol);
			
			// borra la ciudad del camino para probar con otra
			sol.eliminaUltimaCiudad();	
			if (DBG) System.out.println("elimina:"+i);		
		}
	}
}
