package viajante_va;
import java.util.*;

public class Itinerario {
	
	private final double[][] distancias;
	private LinkedList<Integer> camino = new LinkedList<Integer>();
	private double distancia = 0.0;
	
	/**
	 * Crea un itinerario que incluye sólo la ciudad de origen.
	 * @param distancias matriz de distancias entre ciudades
	 * @param ciudadOrigen ciudad comienzo del itinerario
	 */
	public Itinerario(double[][] distancias, int ciudadOrigen) {
		this.distancias = distancias;
		camino.add(ciudadOrigen);
	}
	
	/**
	 * Comprueba si el itinerario es completo (contiene todas las ciudades)
	 * @return verdadero si el itinerario es completo y falso en
	 * caso contrario
	 */
	public boolean recorridoCompleto() {
		return camino.size() == distancias.length;
	}
	
	/**
	 * Incluye la ciudad al final del itinerario.
	 * @param ciudad ciudad a incluir
	 */
	public void incluyeCiudad(int ciudad) {
		assert !camino.contains(ciudad); // no est�n repetidos

		// actualiza la distancia recorrida
		distancia += distancias[camino.getLast()][ciudad];
		// a�ade la ciudad al final del camino
		camino.add(ciudad);
	}
	
	/**
	 * Elimina la última ciudad incluida en el itinerario
	 */
	public void eliminaUltimaCiudad() {
		// elimina la última
		int eliminada = camino.removeLast();
		
		// actualiza la distancia recorrida
		distancia -= distancias[camino.getLast()][eliminada];
	}

	/**
	 * Permite conocer si una ciudad ya ha sido incluida en el itinerario
	 * @param ciudad ciudad a comprobar si está incluida
	 * @return verdadero si la ciudad ya está incluida en
	 * el itinerario y falso en caso contrario
	 */
	public boolean ciudadYaIncluida(int ciudad) {
		return camino.contains(ciudad);
	}
	
	/**
	 * Realiza una copia idéntica al objeto actual
	 * @return copia del objeto actual
	 */
	public Itinerario copia() {
		Itinerario copia = new Itinerario(distancias, camino.getFirst());
		copia.camino = new LinkedList<Integer>(camino);
		copia.distancia = distancia;
		return copia;
	}
	
	/**
	 * Retorna la distancia recorrida en el itinerario
	 * @return distancia recorrida en el itinerario
	 */
	public double distanciaRecorrida() {
		return distancia;
	}
	

	@Override
	public String toString() {
		return camino.toString() + "=>" + distancia;
	}
}
